class PromptConstants:
    OUTPUT_DATA_MODEL = "output_data_model"
    TEMPLATE_STR = "template"
    INPUT_VARIABLES_STR = "input_variables"
    PARTIAL_VARIABLES_STR = "partial_variables"
    TEXT_STR = "text"
    OUTPUT_DATA_MODEL_CLASS_SEPARATOR = "\n\n"
