class AzureOpenAIConstants:
    OPENAI_API_TYPE_STR_NAME: str = "OPENAI_API_TYPE"
    OPENAI_API_BASE_STR_NAME: str = "OPENAI_API_BASE"
    OPENAI_API_VERSION_STR_NAME: str = "OPENAI_API_VERSION"
    OPENAI_DEPLOYMENT_NAME_STR_NAME: str = "OPENAI_DEPLOYMENT_NAME"
    OPENAI_API_KEY_STR_NAME: str = "OPENAI_API_KEY"
    OPENAI_MODEL_NAME_STR_NAME: str = "OPENAI_MODEL_NAME"


class AzureMLOnlineEndpointConstants:
    AZURE_API_KEY_STR_NAME = "AZURE_API_KEY"
    AZURE_ENDPOINT_URL_STR_NAME = "AZURE_ENDPOINT_URL"
    AZURE_DEPLOYMENT_NAME_STR_NAME = "AZURE_DEPLOYMENT_NAME"
