class VertexModelConstants:
    RESPONSE_SEPARATOR = "<LLM_RESPONSE_SEPARATOR>"
    RESPONSE_BLOCKED_STR = "<RESPONSE_BLOCKED>"

    GCP_PROJECT_ID_STR_NAME = "GCP_PROJECT_ID"
    GCP_LLM_REGION_STR_NAME = "GCP_LLM_REGION"
    GCP_MODEL_NAME_STR_NAME = "GCP_MODEL_NAME"
